"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const path = __importStar(require("path"));
const fs = __importStar(require("fs-extra"));
const boa = require('@pipcook/boa');
const tf = boa.import('tensorflow');
const sys = boa.import('sys');
sys.path.insert(0, path.join(__dirname, '..'));
const { train, evaluate } = boa.import('pyscript.index');
const config = tf.compat.v1.ConfigProto();
config.gpu_options.allow_growth = true;
tf.compat.v1.InteractiveSession(boa.kwargs({
    config: config
}));
/**
 * this is plugin used to train tfjs model with pascal voc data format for image classification problem.
 * @param data : train data
 * @param model : model loaded before
 * @param epochs : need to specify epochs
 * @param batchSize : need to specify batch size
 * @param optimizer : need to specify optimizer
 */
const ModelTrain = (data, model, args) => __awaiter(void 0, void 0, void 0, function* () {
    const { epochs = 10, batchSize = 16, modelPath } = args;
    // create tf summary writter
    const trainSummaryWriter = tf.summary.create_file_writer(path.join(modelPath, 'summary'));
    const { trainLoader, validationLoader } = data;
    const count = yield trainLoader.len();
    let valBatchesPerEpoch;
    if (validationLoader) {
        const valCount = yield validationLoader.len();
        valBatchesPerEpoch = Math.floor(valCount / batchSize);
    }
    const batchesPerEpoch = Math.floor(count / batchSize);
    const trainModel = model.model;
    let iterations = 0;
    for (let i = 0; i < epochs; i++) {
        console.log(`Epoch ${i}/${epochs} start`);
        for (let j = 0; j < batchesPerEpoch; j++) {
            const dataBatch = yield data.trainLoader.nextBatch(batchSize);
            train(dataBatch.map((ele) => ele.data), dataBatch.map((ele) => ele.label), trainModel, j, batchesPerEpoch, trainSummaryWriter, iterations++);
        }
        if (validationLoader) {
            let loss = 0;
            let accuracy = 0;
            for (let j = 0; j < valBatchesPerEpoch; j++) {
                const dataBatch = yield validationLoader.nextBatch(batchSize);
                const evaluateRes = evaluate(dataBatch.map((ele) => ele.data), dataBatch.map((ele) => ele.label), trainModel);
                // the value returned by evaluate can be either tensor or number
                // if it's tensor, we need to call tensor.numpy() to get numbers
                if (typeof evaluateRes[0] === 'number') {
                    loss += evaluateRes[0];
                }
                else {
                    loss += Number(evaluateRes[0].numpy());
                }
                if (typeof evaluateRes[1] === 'number') {
                    accuracy += evaluateRes[1];
                }
                else {
                    accuracy += Number(evaluateRes[1].numpy());
                }
            }
            loss /= valBatchesPerEpoch;
            accuracy /= valBatchesPerEpoch;
            console.log(`Validation Result ${i}/${epochs} result --- loss: ${loss} accuracy: ${accuracy}`);
        }
    }
    yield fs.ensureDir(modelPath);
    yield trainModel.save_weights(path.join(modelPath, 'weights.h5'));
    yield trainModel.save(path.join(modelPath, 'model.h5'));
    const result = Object.assign(Object.assign({}, model), { model: trainModel });
    return result;
});
exports.default = ModelTrain;
//# sourceMappingURL=index.js.map